@@ -3,12 +3,14 @@ use crate::zellij_tile::data::{ModeInfo, Palette};
 use crate::{
     os_input_output::{AsyncReader, ChildId, Pid, ServerOsApi},
     thread_bus::Bus,
-    SessionState,
+    ClientId, SessionState,
 };
+use std::convert::TryInto;
 use std::path::PathBuf;
 use std::sync::{Arc, RwLock};
 use zellij_utils::input::command::TerminalAction;
 use zellij_utils::input::layout::LayoutTemplate;
+use zellij_utils::ipc::IpcReceiverWithContext;
 use zellij_utils::pane_size::Size;
 
 use std::os::unix::io::RawFd;
@@ -17,7 +19,6 @@ use zellij_utils::ipc::ClientAttributes;
 use zellij_utils::nix;
 
 use zellij_utils::{
-    errors::ErrorContext,
     interprocess::local_socket::LocalSocketStream,
     ipc::{ClientToServerMsg, ServerToClientMsg},
 };
@@ -26,49 +27,44 @@ use zellij_utils::{
 struct FakeInputOutput {}
 
 impl ServerOsApi for FakeInputOutput {
-    fn set_terminal_size_using_fd(&self, _fd: RawFd, _cols: u16, _rows: u16) {
+    fn set_terminal_size_using_fd(&self, fd: RawFd, cols: u16, rows: u16) {
         // noop
     }
     fn spawn_terminal(&self, _file_to_open: TerminalAction) -> (RawFd, ChildId) {
         unimplemented!()
     }
-    fn read_from_tty_stdout(&self, _fd: RawFd, _buf: &mut [u8]) -> Result<usize, nix::Error> {
+    fn read_from_tty_stdout(&self, fd: RawFd, buf: &mut [u8]) -> Result<usize, nix::Error> {
         unimplemented!()
     }
-    fn async_file_reader(&self, _fd: RawFd) -> Box<dyn AsyncReader> {
+    fn async_file_reader(&self, fd: RawFd) -> Box<dyn AsyncReader> {
         unimplemented!()
     }
-    fn write_to_tty_stdin(&self, _fd: RawFd, _buf: &[u8]) -> Result<usize, nix::Error> {
+    fn write_to_tty_stdin(&self, fd: RawFd, buf: &[u8]) -> Result<usize, nix::Error> {
         unimplemented!()
     }
-    fn tcdrain(&self, _fd: RawFd) -> Result<(), nix::Error> {
+    fn tcdrain(&self, fd: RawFd) -> Result<(), nix::Error> {
+        unimplemented!()
+    }
+    fn kill(&self, pid: Pid) -> Result<(), nix::Error> {
+        unimplemented!()
+    }
+    fn force_kill(&self, pid: Pid) -> Result<(), nix::Error> {
         unimplemented!()
     }
     fn box_clone(&self) -> Box<dyn ServerOsApi> {
         Box::new((*self).clone())
     }
-    fn force_kill(&self, _pid: Pid) -> Result<(), nix::Error> {
+    fn send_to_client(&self, client_id: ClientId, msg: ServerToClientMsg) {
         unimplemented!()
     }
-    fn kill(&self, _pid: Pid) -> Result<(), nix::Error> {
+    fn new_client(
+        &mut self,
+        client_id: ClientId,
+        stream: LocalSocketStream,
+    ) -> IpcReceiverWithContext<ClientToServerMsg> {
         unimplemented!()
     }
-    fn recv_from_client(&self) -> (ClientToServerMsg, ErrorContext) {
-        unimplemented!()
-    }
-    fn send_to_client(&self, _msg: ServerToClientMsg) {
-        unimplemented!()
-    }
-    fn add_client_sender(&self) {
-        unimplemented!()
-    }
-    fn send_to_temp_client(&self, _msg: ServerToClientMsg) {
-        unimplemented!()
-    }
-    fn remove_client_sender(&self) {
-        unimplemented!()
-    }
-    fn update_receiver(&mut self, _stream: LocalSocketStream) {
+    fn remove_client(&mut self, client_id: ClientId) {
         unimplemented!()
     }
     fn load_palette(&self) -> Palette {
@@ -89,7 +85,7 @@ fn create_new_screen(size: Size) -> Screen {
     };
     let max_panes = None;
     let mode_info = ModeInfo::default();
-    let session_state = Arc::new(RwLock::new(SessionState::Attached));
+    let session_state = Arc::new(RwLock::new(SessionState::new()));
     Screen::new(
         bus,
         &client_attributes,
@@ -101,7 +97,7 @@ fn create_new_screen(size: Size) -> Screen {
 }
 
 fn new_tab(screen: &mut Screen, pid: i32) {
-    screen.apply_layout(LayoutTemplate::default().into(), vec![pid]);
+    screen.apply_layout(LayoutTemplate::default().try_into().unwrap(), vec![pid]);
 }
 
 #[test]
